package kayttoliittyma;

import java.awt.GridLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import peli.Paikka;

public class Gui extends JFrame {
    
    private SokkelopelinRajapinta game;
    
    private GridLayout gridLayout;
    private TileWidget[][] tiles;
    
    private ImageLoader imageLoader;
    
    public Gui(SokkelopelinRajapinta game) {
        this.game = game;
        
        this.imageLoader = new ImageLoader();
        
        setLocationByPlatform(true);
        setResizable(false);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setTitle("Ohpesokkelo!");
        
        updateGui();
        
        addKeyListener(new KeyboardHandler());
        
        setVisible(true);
    }
    
    private void updateGui() {
        try {
            int rows = game.sokkelonKorkeus();
            int cols = game.sokkelonLeveys();
            
            updateGridSize(rows, cols);
            
            updateGridImages();
            
        } catch (ProxyException e) {
            showError(e);
        }
    }

    private void updateGridImages() {
        int rows = tiles.length;
        for (int row = 0; row < rows; ++row) {
            int cols = tiles[row].length;
            for (int col = 0; col < cols; ++col) {
                BufferedImage img = imageAt(row, col);
                tiles[row][col].setImage(img);
            }
        }
    }
    
    private BufferedImage imageAt(int row, int col) {
        String imageName = game.kuvaPaikassa(new Paikka(row, col));
        return imageLoader.get(imageName);
    }

    private void updateGridSize(int rows, int cols) {
        if (gridLayout == null || rows != gridLayout.getRows() || cols != gridLayout.getColumns()) {
            
            removeAllTiles(rows, cols);
            
            gridLayout = new GridLayout(rows, cols);
            this.setLayout(gridLayout);
            
            this.setSize(cols * 32, rows * 32); // bad - doesn't account for window borders
            
            createNewTiles(rows, cols);
            
            this.validate();
        }
    }

    private void removeAllTiles(int rows, int cols) {
        if (tiles != null) {
            for (int row = 0; row < rows; ++row) {
                for (int col = 0; col < cols; ++col) {
                    this.remove(tiles[row][col]);
                }
            }
        }
    }
    
    private void createNewTiles(int rows, int cols) {
        tiles = new TileWidget[rows][cols];
        
        for (int row = 0; row < rows; ++row) {
            for (int col = 0; col < cols; ++col) {
                TileWidget tw = new TileWidget();
                tiles[row][col] = tw;
                this.add(tw);
            }
        }
    }
    
    private void showError(Exception e) {
        String msg;
        if (e instanceof ProxyException) {
            msg = e.getMessage();
            if (e.getCause() != null) {
                msg += "\n" + exceptionFullString(e.getCause());
            }
        } else {
            msg = exceptionFullString(e);
        }

        System.err.println(msg);

        JOptionPane.showMessageDialog(this, msg, "Virhe", JOptionPane.ERROR_MESSAGE);
        System.exit(1);
    }

    private String exceptionFullString(Throwable e) {
        StringWriter buf = new StringWriter();
        e.printStackTrace(new PrintWriter(buf));
        return buf.toString();
    }
    
    private class KeyboardHandler extends KeyAdapter {
        @Override
        public void keyPressed(KeyEvent e) {
            try {
                boolean handled = true;
                boolean ok = false;
                
                switch (e.getKeyCode()) {
                case KeyEvent.VK_LEFT: ok = game.yritaVasemmalle(); break;
                case KeyEvent.VK_RIGHT: ok = game.yritaOikealle(); break;
                case KeyEvent.VK_UP: ok = game.yritaYlos(); break;
                case KeyEvent.VK_DOWN: ok = game.yritaAlas(); break;
                default:
                    handled = false;
                }
                
                if (handled) {
                    if (!ok) {
                        System.out.println("Töks.");
                    }
                    e.consume();
                    Gui.this.updateGui();
                }
            } catch (ProxyException ex) {
                showError(ex);
            }
        }
    }
}
