package kayttoliittyma;

import java.awt.image.BufferedImage;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;

import javax.imageio.ImageIO;

public class ImageLoader {
    private ClassLoader cl;
    private HashMap<String, BufferedImage> cache;
    
    public ImageLoader() {
        cl = ClassLoader.getSystemClassLoader();
        cache = new HashMap<String, BufferedImage>();
    }
    
    public BufferedImage get(String name) {
        name = cleanName(name);

        try {
            BufferedImage img = cache.get(name);
            if (img == null) {
                String path = "kuvat/" + name + ".png";
                InputStream is = cl.getResourceAsStream(path);
                if (is == null) {
                    throw new FileNotFoundException(path);
                }
                img = ImageIO.read(is);
                cache.put(name, img);
            }
            return img;
        } catch (IOException e) {
            System.err.println("Kuvan " + name + " lataus epäonnistui :(");
            throw new RuntimeException(e);
        }
    }

    private String cleanName(String name) {
        if (name.endsWith(".png")) {
            name = name.substring(0, name.length() - ".png".length());
        }
        if (name.startsWith("kuvat/")) {
            name = name.substring("kuvat/".length());
        }
        return name;
    }
}
