package kayttoliittyma;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class Proxy<T> {

    protected Object target;
    
    public Proxy(Object target) {
        this.target = target;
    }
    
    @SuppressWarnings("unchecked")
    protected Class<T> targetClass() {
        return (Class<T>)target.getClass();
    }
    
    @SuppressWarnings("unchecked")
    public <R> R callMethod(Class<R> retClass, String methodName, Object ... args) {
        Method method = findMatchingMethod(retClass, methodName, args);
        Object ret = invoke(method, args);
        return (R)ret;
    }

    protected Method findMatchingMethod(Class<?> retClass, String methodName, Object ... args) {
        Class<?>[] argTypes = typesOf(args);
        try {
            Method m = targetClass().getMethod(methodName, argTypes);
            if (m.getReturnType().equals(retClass)) {
                return m;
            } else {
                throw new ProxyException(
                    "Luokan " + targetClass().getSimpleName() +
                    " metodin " + methodName +
                    " paluutyyppi on " + m.getReturnType().getSimpleName() +
                    ". Sen pitäisi olla " + retClass.getSimpleName() + ".");
            }
        } catch (NoSuchMethodException e) {
            String methodDesc = methodDesc(retClass, methodName, argTypes);
            throw new ProxyException("Puuttuva metodi 'public " + methodDesc + "' luokassa " + targetClass().getSimpleName());
        }
        
    }

    protected Class<?>[] typesOf(Object[] args) {
        Class<?>[] types = new Class<?>[args.length];
        for (int i = 0; i < args.length; ++i) {
            types[i] = preferPrimitive(args[i].getClass());
        }
        return types;
    }
    
    protected Class<?> preferPrimitive(Class<?> cls) {
        if (cls.equals(Byte.class)) {
            return Byte.TYPE;
        } else if (cls.equals(Character.class)) {
            return Character.TYPE;
        } else if (cls.equals(Short.class)) {
            return Short.TYPE;
        } else if (cls.equals(Integer.class)) {
            return Integer.TYPE;
        } else if (cls.equals(Long.class)) {
            return Long.TYPE;
        } else if (cls.equals(Float.class)) {
            return Float.TYPE;
        } else if (cls.equals(Double.class)) {
            return Double.TYPE;
        } else {
            return cls;
        }
    }

    protected String methodDesc(Class<?> retClass, String methodName, Class<?>[] argTypes) {
        String desc = retClass.getSimpleName() + " " + methodName + "(";
        if (argTypes.length > 0) {
            for (int i = 0; i < argTypes.length - 1; ++i) {
                desc += argTypes[i].getSimpleName() + ", ";
            }
            desc += argTypes[argTypes.length - 1].getSimpleName();
        }
        desc += ")";
        return desc;
    }

    protected Object invoke(Method method, Object ... args) {
        try {
            return method.invoke(target, args);
        } catch (InvocationTargetException e) {
            throw new ProxyException("Metodikutsussa tapahtui virhe", e.getCause());
        } catch (Exception e) {
            throw new ProxyException("Metodikutsussa tapahtui virhe", e);
        }
    }
    
}
