package peli;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

public class Kartta {

    private char[][] merkit;

    public static Kartta lueTiedostosta(String tiedosto) {
        InputStream virta = ClassLoader.getSystemResourceAsStream(tiedosto);
        if (virta == null) {
            throw new RuntimeException("Karttatiedostoa '" + tiedosto + "' ei löytynyt");
        }
        return new Kartta(virta);
    }

    public static Kartta lueMerkkijonosta(String mjono) {
        return new Kartta(new ByteArrayInputStream(mjono.getBytes()));
    }

    public Kartta(InputStream lukuvirta) {
        Scanner lukija = new Scanner(lukuvirta);
        ArrayList<String> rivit = lueRivit(lukija);
        tarkistaRivit(rivit);
        this.merkit = teeTaulukko(rivit);
    }

    public int leveys() {
        return merkit[0].length;
    }

    public int korkeus() {
        return merkit.length;
    }

    public char merkkiKohdassa(Paikka p) {
        return merkkiKohdassa(p.rivi(), p.sarake());
    }

    public char merkkiKohdassa(int rivi, int sarake) {
        return merkit[rivi][sarake];
    }

    @Override
    public String toString() {
        String tulos = "";
        for (int i = 0; i < merkit.length; ++i) {
            tulos += new String(merkit[i]) + "\n";
        }
        return tulos;
    }

    private ArrayList<String> lueRivit(Scanner lukija) {
        ArrayList<String> rivit = new ArrayList<String>();
        while (lukija.hasNextLine()) {
            String rivi = lukija.nextLine().trim();
            if (!rivi.isEmpty()) {
                rivit.add(rivi);
            }
        }
        return rivit;
    }

    private void tarkistaRivit(List<String> rivit) {
        if (rivit.isEmpty()) {
            throw new IllegalArgumentException("Kartalla ei ollut yhtään riviä");
        }
        
        int pituus = rivit.get(0).length();
        for (int i = 0; i < rivit.size(); ++i) {
            String rivi = rivit.get(i);
            if (rivi.length() != pituus) {
                throw new IllegalArgumentException("Kartassa virhe rivillä " + (i+1));
            }
        }
    }

    private char[][] teeTaulukko(ArrayList<String> rivit) {
        int korkeus = rivit.size();
        int leveys = rivit.get(0).length();
        char[][] taulukko = new char[korkeus][leveys];
        for (int i = 0; i < korkeus; ++i) {
            String rivi = rivit.get(i);
            for (int j = 0; j < leveys; ++j) {
                taulukko[i][j] = rivi.charAt(j);
            }
        }
        return taulukko;
    }

}
